#include "wifinetworkselector.h"
#include "archprofileeditor.h"
#include "netutils.h"
#include <QDebug>
#include <QDialog>
#include <QProcess>
#include <QTextStream>
#include <QTreeWidget>
#include <QFile>
#include <QMap>
#include <QMessageBox>
#include <QIcon>
#include <QHBoxLayout>
#include <QVBoxLayout>
#include <QPushButton>
#include <QSettings>
#include <QFileDialog>
#include <QPalette>
//
ArchProfileEditor::ArchProfileEditor(QWidget * parent, Qt::WFlags f, QString profileFile)
	: QDialog(parent, f)
{
	netUtils = new NetUtils();
	setupUi(this);
	setWindowIcon(QIcon(":/archlogo.png"));
	
	QPushButton *myOkButton = buttonBox->button(QDialogButtonBox::Ok);
	myOkButton->setIcon(QIcon(":/ok.png"));
	myOkButton->setText(tr("OK"));
	QPushButton *myCancelButton = buttonBox->button(QDialogButtonBox::Cancel);
	myCancelButton->setIcon(QIcon(":/fileclose.png"));
	myCancelButton->setText(tr("Cancel"));
	
	cbWifiScan->setItemData(0, QString("yes"));
	cbWifiScan->setItemData(1, QString("no"));
	cbWifiAutoconnect->setItemData(0, QString("yes"));
	cbWifiAutoconnect->setItemData(1, QString("no"));
	//cbInterfaceName->setToolTip(netUtils->getVendorNameForInterface("eth0"));
	
	//connect( checkUseDhcp, SIGNAL( stateChanged(int) ), this, SLOT( useDhcpStateChanged(int) ) );
	connect( gbUseDhcp, SIGNAL( toggled(bool) ), this, SLOT( useDhcpStateChanged(bool) ) );
	connect( pbChooseEssid, SIGNAL( clicked() ), this, SLOT( pbChooseEssidClicked() ) );
	connect( pbWpaConf, SIGNAL( clicked() ), this, SLOT( chooseWpaConfFileName() ) );
	connect( cbInterfaceName, SIGNAL( currentIndexChanged(const QString&) ), this, SLOT( cbInterfaceNameStateChanged(const QString&) ) );
	connect( cbWirelessMode, SIGNAL( currentIndexChanged(const QString&) ), this, SLOT( cbWirelessModeStateChanged(const QString&) ) );
	connect( leIp, SIGNAL( textChanged(const QString&) ), this, SLOT( updateBroadcastAdress(const QString&) ) );
	connect( leProfileName, SIGNAL( textChanged(const QString&) ), this, SLOT( checkLeProfileNameSyntax(const QString&) ) );
	connect( buttonBox, SIGNAL( accepted() ), this, SLOT( profileGenerator() ) );
	connect( buttonBox, SIGNAL( rejected() ), this, SLOT( quitDialog() ) );

	QPalette p(leProfileName->palette());
	//p.setColor(QPalette::Base,Qt::cyan);
	p.setColor(QPalette::Base,QColor(224,224,254,255));
	leProfileName->setPalette(p);
		
	if ( profileFile.isEmpty() )
	{
		leProfileName->setFocus();
		editionMode = "new";
		initNewProfile();
	}
	else
	{
		editionMode = "edit";
		initEditedProfile(profileFile);
	}
	
}

QMap<QString, QString> ArchProfileEditor::getProfileKeys(QString file, QString charComment)
{
	QStringList infosList;
	QMap<QString, QString> profileMap;

	if ( QFile::exists(file) )
	{
		QSettings settings(file, QSettings::IniFormat);
		infosList = settings.allKeys();

		for (int i=0 ; i < infosList.size() ; ++i)
		{
			//profileMap[infosList.at(i)] = settings.value(infosList.at(i), "").toString();
			
			if (!infosList.at(i).startsWith(charComment))
			{
				profileMap[infosList.at(i)] = settings.value(infosList.at(i), "").toString();
				if (!charComment.isEmpty())
					profileMap[infosList.at(i)] = profileMap[infosList.at(i)].left(profileMap[infosList.at(i)].lastIndexOf(charComment)).trimmed(); // remove commentary
			}
		}
	}

	return profileMap;
}

void ArchProfileEditor::initNewProfile()
{
	// Page 1 : General
	leHostName->setText(getCurrentHostName());

	//cbInterfaceName->addItems(getAvailableInterfaces());
	QStringList availableInterfaces = getAvailableInterfaces();

	QString iconName;
	for (int i = 0; i < availableInterfaces.size(); ++i)
	{
		QStringList interfaceDetails = availableInterfaces.at(i).split("/");
		if (interfaceDetails.at(0).contains("WIFI"))
			iconName = ":/wifi_radar_22x22.png";
		else
			iconName = ":/theme2_none.png";
		cbInterfaceName->addItem(QIcon(iconName), QString("%1 (%2)").arg(interfaceDetails.at(1)).arg(interfaceDetails.at(0)));
	}
	
	QStringList interfaceDetails = availableInterfaces.at(0).split("/");
	interfaceName = interfaceDetails.at(1);

	// Page 2 : Ip & Dns
	//checkUseDhcp->setChecked(true);
	gbUseDhcp->setChecked(true);
	gbIp->setDisabled(true);
	gbDns->setDisabled(true);
	
	// Page 3 : Wireless
	cbWifiScan->setCurrentIndex(1); // set to 'no' as a default
	if (availableInterfaces.at(0).contains("(ETHERNET)"))
	{
		tabWidget->setTabEnabled(2, false);//tabWidget->widget(2)->setDisabled(true);
		spWifiWait->setValue(15);
	}
}

void ArchProfileEditor::initEditedProfile(QString file)
{
	QString profileFileName = netUtils->profilesDirName + "/" + file;

	if ( !QFile::exists(profileFileName) )
	{
		QMessageBox::warning(this, tr("Application"),
							tr("Cannot read file '%1'.").arg(profileFileName)
							);
		return;
	}
	else
	{
		QStringList availableInterfaces = getAvailableInterfaces();
		QString iconName;
		for (int i = 0; i < availableInterfaces.size(); ++i)
		{
			QStringList interfaceDetails = availableInterfaces.at(i).split("/");
			if (interfaceDetails.at(0).contains("WIFI"))
				iconName = ":/wifi_radar_22x22.png";
			else
				iconName = ":/theme2_none.png";
			cbInterfaceName->addItem(QIcon(iconName), QString("%1 (%2)").arg(interfaceDetails.at(1)).arg(interfaceDetails.at(0)));
		}

		QMap<QString, QString> profileKey = getProfileKeys(profileFileName, "#");
		
// 		qDebug() << "profile KEY INTERFACE=" << qPrintable(profileKey["INTERFACE"]) << endl;
// 		qDebug() << "profile KEY DESCRIPTION=" << qPrintable(profileKey["DESCRIPTION"]) << endl;
// 		qDebug() << "profile KEY HOSTNAME=" << qPrintable(profileKey["HOSTNAME"]) << endl;
// 
// 		QMessageBox::warning(this, tr("Application"),
// 							 tr("profile KEY INTERFACE='%1'.").arg(profileKey["INTERFACE"])
// 							);


		// General Settings
		leProfileName->setText(file);
		leProfileName->setEnabled(false);
		leDescription->setText(profileKey["DESCRIPTION"]);
		leHostName->setText(profileKey["HOSTNAME"]);

		if ( !profileKey["INTERFACE"].isEmpty() )
		{
			for (int i=0 ; i < cbInterfaceName->count() ; ++i)
			{
				if ( cbInterfaceName->itemText(i).startsWith(profileKey["INTERFACE"].toLower()) )
				{
					cbInterfaceName->setCurrentIndex(i);
					break;
				}
			}
		}

		// Ip & Dns Settings
		if (profileKey["IFOPTS"].toLower() == "dhcp" || profileKey["IP"].toLower() == "dhcp")
		{
			gbUseDhcp->setChecked(true);//checkUseDhcp->setChecked(true);
			gbIp->setDisabled(true);
			gbDns->setDisabled(true);
			spDhcpTimeout->setValue(profileKey["DHCP_TIMEOUT"].toInt());
		}
		else
		{
			gbUseDhcp->setChecked(false);//checkUseDhcp->setChecked(false);
			gbIp->setDisabled(false);
			gbDns->setDisabled(false);
			
			QStringList ipDetails = profileKey["IFOPTS"].simplified().split(" ");

			if (ipDetails.size() > 4)
			{
				leIp->setText(ipDetails.at(0));
				leBroadcast->setText(ipDetails.at(4));

				for (int i=0 ; i < cbNetmask->count() ; ++i)
				{
					if (ipDetails.at(2) == cbNetmask->itemText(i))
					{
						cbNetmask->setCurrentIndex(i);
						break;
					}
				}
			}
			
			leGateway->setText(profileKey["GATEWAY"]);
			leDomain->setText(profileKey["DOMAIN"]);
			leDns1->setText(profileKey["DNS1"]);
			leDns2->setText(profileKey["DNS2"]);
		}

		// Wireless Settings
		if ( profileKey["CONNECTION"] == "wireless" ) // && profileKey["ESSID"].trimmed() != "")
		{
			//tabWidget->widget(2)->setDisabled(false);
			tabWidget->setTabEnabled(2, true);
			
			if (profileKey["SECURITY"].toLower() == "wep")
				cbWirelessMode->setCurrentIndex(0);
			else if (profileKey["SECURITY"].toLower() == "wpa")
				cbWirelessMode->setCurrentIndex(1);
			else if (profileKey["SECURITY"].toLower() == "wpa-config")
				cbWirelessMode->setCurrentIndex(2);
			else //if (profileKey["SECURITY"].toLower() == "none")
				cbWirelessMode->setCurrentIndex(3);

			leEssid->setText(profileKey["ESSID"]);
			leKey->setText(profileKey["KEY"]);
			spWifiWait->setValue(profileKey["TIMEOUT"].toInt());
			
			if (profileKey["SCAN"].toLower() == "yes")
				cbWifiScan->setCurrentIndex(0);
			else
				cbWifiScan->setCurrentIndex(1);
			
			if (profileKey["AUTOCONNECT"].toLower() == "no")
				cbWifiAutoconnect->setCurrentIndex(1);
			else
				cbWifiAutoconnect->setCurrentIndex(0);
			
			QStringList wifiDetails = profileKey["IWOPTS"].simplified().split(" ");
			
			if (wifiDetails.size() > 5)
			{
				gbWirelessSetupAdvanced->setChecked(true);
				
				for (int i=0 ; i < cbChannel->count() ; ++i)
				{
					if (wifiDetails.at(5) == cbChannel->itemText(i))
					{
						cbChannel->setCurrentIndex(i);
						break;
					}
				}

				if (wifiDetails.at(1).toLower() == "managed")
					cbMode->setCurrentIndex(0);
				else
					cbMode->setCurrentIndex(1);
			}

			// WPA Specific Settings
			if (profileKey["SECURITY"].toLower().startsWith("wpa"))
			{
				//cbWirelessMode->setCurrentIndex(1);
				gbWpaOptions->setDisabled(false);
				leWpaConf->setText(profileKey["WPA_CONF"]);
				leWpaOpts->setText(profileKey["WPA_OPTS"]);
			}
			else
			{
				//cbWirelessMode->setCurrentIndex(0);
				gbWpaOptions->setDisabled(true);
			}
		}
		else
			tabWidget->setTabEnabled(2, false);//tabWidget->widget(2)->setDisabled(true);
	}
}


void ArchProfileEditor::chooseWpaConfFileName()
{
	QString fileName = QFileDialog::getOpenFileName(
											this,
											tr("Choose a wpa-conf file"),
											QDir::homePath()
										   );
	if (!fileName.trimmed().isEmpty())
		leWpaConf->setText(fileName);
}

void ArchProfileEditor::pbChooseEssidClicked()
{
	WifiNetworkSelector *essidDlg = new WifiNetworkSelector(this);
	essidDlg->exec();
	//qDebug(qPrintable(essidDlg->essidSelected));
	if (!essidDlg->essidSelected.isEmpty())
	{
		leEssid->setText(essidDlg->essidSelected);
		leEssid->setToolTip(essidDlg->mac_addressSelected);
	}
		
		
	if (!essidDlg->channelSelected.isEmpty())
	{
		for (int i=0 ; i < cbChannel->count() ; ++i)
		{
			if (essidDlg->channelSelected == cbChannel->itemText(i))
			{
				cbChannel->setCurrentIndex(i);
				break;
			}
		}
	}

	if (essidDlg->modeSelected.isEmpty() || essidDlg->modeSelected == "Managed" )
		cbMode->setCurrentIndex(0); // managed or empty
	else
		cbMode->setCurrentIndex(1); // ad-hoc

	if (essidDlg->protocolSelected == "WPA")
		cbWirelessMode->setCurrentIndex(1);//cbWirelessModeStateChanged("WPA");
	else
		cbWirelessMode->setCurrentIndex(0);
	
	//delete essidDlg;
}

void ArchProfileEditor::cbInterfaceNameStateChanged(const QString &text)
{
	
	// Disable Wireless Tab if LAN ethernet device selected
	if (text.contains("(ETHERNET)"))
		tabWidget->setTabEnabled(2, false);//tabWidget->widget(2)->setDisabled(true);
	else
		tabWidget->setTabEnabled(2, true);//tabWidget->widget(2)->setDisabled(false);
	
	// Display tooltip with network card manufacturer name. 
	QStringList vendorName = text.split(" ");
	if (vendorName.size() > 0)
		cbInterfaceName->setToolTip(netUtils->getVendorNameForInterface(vendorName.at(0)));
	else
		cbInterfaceName->setToolTip(netUtils->getVendorNameForInterface(""));
}

void ArchProfileEditor::cbWirelessModeStateChanged(const QString &text)
{
	if (text == "WEP" || text == "None")
		gbWpaOptions->setDisabled(true);
	else
		gbWpaOptions->setDisabled(false);
}

void ArchProfileEditor::useDhcpStateChanged(bool state)//(int state)
{
	//QString str;
	//str.setNum(state);
	switch (state)
	{
		case false://case 0:
			gbIp->setDisabled(false);
			gbDns->setDisabled(false);
			break;
		case true://case 2:
			gbIp->setDisabled(true);
			gbDns->setDisabled(true);
			break;
		default:
			// should never be reached
			break;
	}
}

QString ArchProfileEditor::getCurrentHostName()
{
	return netUtils->getHostname();
// 	QString str = "";
// 	QString file = "/etc/rc.conf";
// 	
// 	if ( QFile::exists(file) )
// 	{
// 		QSettings settings(file, QSettings::IniFormat);
// 		str = settings.value("HOSTNAME", "").toString();
// 		str = str.left(str.lastIndexOf("#")).trimmed(); // remove commentary
// 	}
// 
// 	return str;
}

QStringList ArchProfileEditor::getAvailableInterfaces()
{
	QStringList availableInterfaces;
	
	QString wifiInterfaceName = netUtils->getWifiInterfaceName();
	QStringList interfacesList = netUtils->getInterfacesList();
	
	for (int i = 0 ; i < interfacesList.size() ; i++)
	{
		//if (interfacesList.at(i) == wifiInterfaceName)
		if (netUtils->getInterfaceWifiStatus(interfacesList.at(i)))
			availableInterfaces << QString("WIFI/%1").arg(interfacesList.at(i));
		else
			availableInterfaces << QString("ETHERNET/%1").arg(interfacesList.at(i));
	}
	
	return availableInterfaces;
}

void ArchProfileEditor::updateBroadcastAdress(const QString & ipText)
{
	QStringList bcastList = ipText.split(".");
	if (bcastList.size() == 4)
	{
		leBroadcast->setText(
							QString("%1.%2.%3.255")
							.arg(bcastList.at(0))
							.arg(bcastList.at(1))
							.arg(bcastList.at(2))
							);
	}
}
/*
QString ArchProfileEditor::checkProfileNameSyntax(const QString & profileText)
{
	QString allowedChars="0123456789_-.AZERTYUIOPQSDFGHJKLMWXCVBNazertyuiopqsdfghjklmwxcvbn";
	QString cleanText = "";//QString cleanText = profileText;
	
	if ( profileText.size() > 0 )
	{
		//if ( !allowedChars.contains( profileText.right(1) ) )
		//	cleanText = profileText.left( profileText.size()-1 );
		for (int i = 0; i < profileText.size(); i++)
		{
			if ( allowedChars.contains( profileText.at(i) ) )
				cleanText.append( profileText.at(i) );
		}
	}
	
	return cleanText;
}
*/

void ArchProfileEditor::checkLeProfileNameSyntax(const QString & profileText)
{
	//QString allowedChars="0123456789_-.AZERTYUIOPQSDFGHJKLMWXCVBNazertyuiopqsdfghjklmwxcvbn";
	if ( profileText.size() > 0 )
	{
		//if ( !allowedChars.contains( profileText.right(1) ) )
			//leProfileName->setText( profileText.left( profileText.size()-1 ) );
			leProfileName->setText( netUtils->checkProfileNameSyntax(profileText) );//leProfileName->setText( checkProfileNameSyntax(profileText) );
	}

}

void ArchProfileEditor::quitDialog()
{
	switch (
			QMessageBox::question(
							this,
							tr("Confirm Editor Exit"),
							tr("Quit without saving ?"),
							tr("&Ok"), tr("&Cancel"),
							0, 1)
			)
	{
		case 1:
			return;
	}
	
	reject();
}

void ArchProfileEditor::profileGenerator()
{
	QString profileNameCleaned = leProfileName->text();
	profileNameCleaned.remove(QChar(' '), Qt::CaseInsensitive);
	QString filePath = netUtils->profilesDirName + "/" + profileNameCleaned;

	QFile file(filePath);
	
	if (leProfileName->text().trimmed().length() < 1)
	{
		QMessageBox::warning(
							this,
							tr("Application"),
							tr("Profile name field is empty !\nPlease choose a name for current profile.")
							);
		tabWidget->setCurrentIndex(0);
		leProfileName->setFocus();
		return;
	}
	
	//if ( tabWidget->isTabEnabled(2) && ( leEssid->text().trimmed().length() < 1 || leKey->text().trimmed().length() < 1 ) )
	if ( tabWidget->isTabEnabled(2) && leEssid->text().trimmed().length() < 1 )
	{
		QMessageBox::warning(
					this,
					tr("Application"),
					   tr("Essid field can't be left blank.\n\nPlease fill it before saving the profile, or choose an ethernet device.")//tr("Essid and Key fields can't be left blank.\n\nPlease fill them before saving the profile.")
							);
		return;
	}

	if ( QFile::exists( filePath ) )
	{
		if ( editionMode == "new" ) // new profile mode
		{
			switch (
					QMessageBox::question(
						this,
						tr("Write Profile Confirmation - Network Profile Editor"),
						tr("Another profile has same name. Overwrite existing profile '%1' ?").arg(leProfileName->text()),
						tr("&Ok"), tr("&Cancel"),
						0, 1)
				)
			{
				case 1:
					return;
			}
		}
		else // profile edition mode
		{
			switch (
					QMessageBox::question(
							this,
							tr("Write Profile Confirmation - Network Profile Editor"),
							tr("Overwrite existing profile '%1' ?").arg(leProfileName->text()),
							tr("&Ok"), tr("&Cancel"),
							0, 1)
				   )
			{
				case 1:
					return;
			}
		}
	}
	
	if (!file.open(QFile::WriteOnly | QFile::Text))
	{
		QMessageBox::warning(
				this,
				tr("Application"),
				tr("Cannot write file '%1'.").arg(filePath)
				);
		return;
	}
	
	QStringList fileBits;
	QString str;
	//QTextStream fileBits(&file);

	fileBits << "#";
	fileBits << "# Network Profile";
	fileBits << "#";
	if (tabWidget->isTabEnabled(2))
		fileBits << "CONNECTION=\"wireless\"";
	else
		fileBits << "CONNECTION=\"ethernet\"";
	fileBits << "DESCRIPTION=\"" + leDescription->text() + "\"";
	fileBits << "# Network Settings";
	QStringList interfaceDetails = cbInterfaceName->currentText().split(" ");
	fileBits << "INTERFACE=" + interfaceDetails.at(0);
	fileBits << "HOSTNAME=\"" + leHostName->text() + "\"";

	fileBits << "# Interface Settings";
	if (gbUseDhcp->isChecked())//if (checkUseDhcp->isChecked())
	{
		fileBits << "IP=\"dhcp\"";
		fileBits << "IFOPTS=\"dhcp\"";
		if ( spDhcpTimeout->value() != 0 )//if ( spWifiWait->value() != 0 )
		{
			str = "";//QString str;
			fileBits << "DHCP_TIMEOUT=" + str.setNum(spDhcpTimeout->value());//fileBits << "DHCP_TIMEOUT=" + str.setNum(spWifiWait->value());
		}
	}
	else
	{
		fileBits << "IP=\"static\"";
		fileBits << "IFOPTS=\"" + leIp->text() + " netmask " + cbNetmask->currentText() + " broadcast " + leBroadcast->text() + "\"";
		fileBits << "GATEWAY=" + leGateway->text();
		fileBits << "# DNS Settings";
		if (!leDomain->text().trimmed().isEmpty())
			fileBits << "DOMAIN=" + leDomain->text();
		fileBits << "DNS1=" + leDns1->text();
		fileBits << "DNS2=" + leDns2->text();
	}
	
	if (!leEssid->text().trimmed().isEmpty()) // Wireless options if applies
	{
		fileBits << "# Wireless Settings";
		fileBits << "SECURITY=\"" + cbWirelessMode->currentText().toLower() + "\"";
		fileBits << "ESSID=\"" + leEssid->text() + "\"";
		fileBits << "KEY=\"" + leKey->text() + "\"";
		//fileBits << "SCAN=\"" + cbWifiScan->currentText() + "\"";
		fileBits << "SCAN=\"" + cbWifiScan->itemData(cbWifiScan->currentIndex()).toString() + "\"";
		
		if ( spWifiWait->value() != 0 )
		{
			str="";//QString str;
			fileBits << "TIMEOUT=" + str.setNum(spWifiWait->value());
		}

		//if ( cbWifiAutoconnect->currentText() == "no" )
		if ( cbWifiAutoconnect->itemData(cbWifiAutoconnect->currentIndex()).toString() == "no" )
		{
			fileBits << "AUTOCONNECT=\"no\"";
		}
		
		if ( !leEssid->toolTip().isEmpty() )
			fileBits << "MAC_ADDRESS=\"" + leEssid->toolTip().toLower() + "\"";
		
		//if (cbChannel->currentText() != "auto" && cbKeyMode->currentText() != "managed")
		if ( gbWirelessSetupAdvanced->isChecked() )
		{
			fileBits << "IWOPTS=\"mode " + cbMode->currentText()
					+ " essid $ESSID channel " + cbChannel->currentText()
					+ " key " + cbKeyMode->currentText()
					+ " $KEY" + "\"";
		}
	
		//if (cbWirelessMode->currentText() == "WPA") // WPA options if applies
		if (cbWirelessMode->currentText().toLower().startsWith("wpa")) // WPA options if applies
		{
			fileBits << "# WPA Specific Settings";
			fileBits << "WPA_OPTS=\"" + leWpaOpts->text() + "\"";
			fileBits << "WPA_CONF=\"" + leWpaConf->text() + "\"";
		}
// 		if (cbWirelessMode->currentText() == "WPA-CONFIG") // WPA-CONFIG options if applies
// 		{
// 			fileBits << "# WPA Specific Settings";
// 			fileBits << "WPA_CONF=\"" + leWpaConf->text() + "\"";
// 		}
	}

	// fill text file
	QTextStream in(&file);
	
	for (int i=0 ; i < fileBits.count() ; ++i)
		in << fileBits.at(i) << endl;//qDebug() << qPrintable(fileBits.at(i)) << endl;

	file.close();
	accept();
}
//
