#include "texteditor.h"
#include "archprofilemanager.h"
#include "archprofileeditor.h"
#include "netutils.h"
#include <QtGui>

#include <iostream>
using namespace std;

//ArchProfileManager::ArchProfileManager( QObject * parent ) : QDialog(parent)
ArchProfileManager::ArchProfileManager() : QDialog()
{
	setModal(true);
	//setSizeGripEnabled(true);
	setWindowTitle ( tr("Arch Assistant - Manage Profiles") );
	setWindowIcon(QIcon(":/archlogo.png"));
	setMinimumSize(630, 320);
	
	treeWidget = new QTreeWidget(this);
	treeWidget->setColumnCount(3);
	treeWidget->setRootIsDecorated(false);
	treeWidget->setAlternatingRowColors(true);
	
	netUtils = new NetUtils();
	//QString profilePath = "/etc/network-profiles/";
	//QString profilePath = netUtils->profilesDirName + "/";
	
	profilesWereUpdated = false;
	profileConnectRequested = false;
	//profileDisconnectRequested = false;
	profileDisconnectRequested = "";
	profileCurrentName="";
	
	QStringList headers;
	headers << trUtf8("Profile Name") << trUtf8("Interface Name") << trUtf8("Interface Type") << trUtf8("State");
	treeWidget->setHeaderLabels(headers);

	QPushButton *pbClose = new QPushButton(trUtf8("&Close"), this);
	QPushButton *pbEdit = new QPushButton(trUtf8("&Edit"), this);
	QPushButton *pbDelete = new QPushButton(trUtf8("&Delete"), this);
	QPushButton *pbNew = new QPushButton(trUtf8("&New"), this);
	QPushButton *pbRename = new QPushButton(trUtf8("&Rename"), this);
	QPushButton *pbCopy = new QPushButton(trUtf8("Copy"), this);
	pbConnect = new QPushButton(trUtf8("Connect"), this);	
	pbDisconnect = new QPushButton(trUtf8("Disconnect"), this);
	QFont font = pbConnect->font();
	font.setBold(true);
	pbConnect->setFont(font);
	pbConnect->setDisabled(true);
	pbDisconnect->setDisabled(true);
	
	pbDelete->setIcon(QIcon(":/cancel.png"));
	pbEdit->setIcon(QIcon(":/edit.png"));
	pbNew->setIcon(QIcon(":/filenew.png"));
	pbClose->setIcon(QIcon(":/fileclose.png"));
	pbRename->setIcon(QIcon(":/editpaste.xpm"));
	pbCopy->setIcon(QIcon(":/editcopy.xpm"));
	
	QVBoxLayout *layoutV = new QVBoxLayout;
	layoutV->addStretch();
	layoutV->addWidget( pbNew );
	layoutV->addWidget( pbEdit );
	layoutV->addWidget( pbDelete );
	layoutV->addWidget( pbRename );
	layoutV->addWidget( pbCopy );
	layoutV->addStretch();
	layoutV->addWidget( pbConnect );
	layoutV->addWidget( pbDisconnect );
	layoutV->addStretch();
	layoutV->addWidget( pbClose );
	
	QHBoxLayout *layout = new QHBoxLayout;
	layout->addWidget( treeWidget );
	layout->addLayout( layoutV );

	//connect( pbClose, SIGNAL( clicked() ), this, SLOT( reject() ) );
	connect( pbClose, SIGNAL( clicked() ), this, SLOT( close() ) );
	connect( pbEdit, SIGNAL( clicked() ), this, SLOT( profileEdit() ) );
	connect( pbRename, SIGNAL( clicked() ), this, SLOT( profileRename() ) );
	connect( pbNew, SIGNAL( clicked() ), this, SLOT( profileNew() ) );
	connect( pbDelete, SIGNAL( clicked() ), this, SLOT( profileDelete() ) );
	connect( pbCopy, SIGNAL( clicked() ), this, SLOT( profileCopy() ) );
	connect( pbConnect, SIGNAL( clicked() ), this, SLOT( profileConnect() ) );
	connect( pbDisconnect, SIGNAL( clicked() ), this, SLOT( profileDisconnect() ) );
	connect( treeWidget, SIGNAL( itemDoubleClicked ( QTreeWidgetItem*, int ) ), this, SLOT( treeItemClicked() ) );
	connect( treeWidget, SIGNAL( itemSelectionChanged() ), this, SLOT( treeItemSelectionChanged() ) );
	//connect( treeWidget, SIGNAL( currentItemChanged(QTreeWidgetItem *, QTreeWidgetItem *) ), this, SLOT( treeItemClicked() ) );
	
	setLayout(layout);

	profileListUpdate();
}
void ArchProfileManager::treeItemSelectionChanged()
{
	QStringList activeProfilesList = netUtils->getActiveProfilesList();
	
	if ( activeProfilesList.contains( treeWidget->currentItem()->text(0) ) )
	{
		pbConnect->setDisabled(true);
		pbDisconnect->setDisabled(false);
	}
	else
	{
		pbConnect->setDisabled(false);
		pbDisconnect->setDisabled(true);
	}
		
	//if (profileCurrentName.isEmpty())
		//pbConnect->setDisabled(false);
		
	profileCurrentName = treeWidget->currentItem()->text(0);
}

void ArchProfileManager::treeItemClicked()
{
	profileEdit();
// 	QMessageBox::warning(this, tr("My Application"),
//                   treeWidget->currentItem()->text(0),
//                    QMessageBox::Save | QMessageBox::Discard
//                    | QMessageBox::Cancel,
//                    QMessageBox::Save);
}

QMap<QString, QString> ArchProfileManager::profileGetInfosForProfile(const QString & file)
{
	QStringList infosList;
	QMap<QString, QString> profileMap;

	if ( QFile::exists(file) )
	{
		QSettings settings(file, QSettings::IniFormat);
		infosList = settings.allKeys();

		for (int i=0 ; i < infosList.size() ; ++i)
		{
			if (!infosList.at(i).startsWith("#"))
			{
				profileMap[infosList.at(i)] = settings.value(infosList.at(i), "").toString();
				profileMap[infosList.at(i)] = profileMap[infosList.at(i)].left(profileMap[infosList.at(i)].lastIndexOf("#")).trimmed(); // remove commentary
			}
		}
	}

	return profileMap;
}

void ArchProfileManager::profileEdit()
{
	if (treeWidget->topLevelItemCount() < 1)
		return;
		
	//TextEditor * te = new TextEditor(QString("/etc/network-profiles/%1").arg(treeWidget->currentItem()->text(0)));
	//custom signal
	//connect(te, SIGNAL(yo()), SLOT( coucou()));
	//te->show();
	ArchProfileEditor * ape = new ArchProfileEditor(this, 0, treeWidget->currentItem()->text(0));

	switch (ape->exec())
	{
		case QDialog::Accepted:
			// yes was clicked
			profileListUpdate();
			break;
		case QDialog::Rejected:
			//return;
			break;
		default:
			// should never be reached
			break;
	}
}

void ArchProfileManager::profileDelete()
{
	if (treeWidget->topLevelItemCount() < 1)
		return;
		
	QMessageBox msgBox;
	msgBox.setWindowTitle(trUtf8("Arch Assistant - Manage profiles"));
	msgBox.setText(trUtf8("Delete profile '%1' ?").arg(treeWidget->currentItem()->text(0)));
	msgBox.setIcon(QMessageBox::Question);
	
	QPushButton *connectButton = msgBox.addButton(tr("Delete profile"), QMessageBox::YesRole);
	QPushButton *abortButton = msgBox.addButton(tr("Cancel"), QMessageBox::NoRole);
	connectButton->setIcon(QIcon(":/cancel.png"));
	abortButton->setIcon(QIcon(":/fileclose.png"));
	//msgBox.addButton(tr("Delete profile"), QMessageBox::YesRole);
	//msgBox.addButton(tr("Cancel"), QMessageBox::NoRole);
	//msgBox.setStandardButtons(QMessageBox::Yes | QMessageBox::No);
	
	msgBox.exec();
	if (msgBox.clickedButton() == abortButton)
		return;

// 	switch (msgBox.exec())
// 	{
// 		case QMessageBox::YesRole:
// 			// yes was clicked
// 			break;
// 		case QMessageBox::NoRole:
// 			return;
// 			break;
// 		default:
// 			// should never be reached
// 			break;
// 	}

	bool okRemove = QFile::remove ( QString("%1/%2").arg(netUtils->profilesDirName).arg(treeWidget->currentItem()->text(0)) );
	if (okRemove)
	{
		cout << "profile removed ok" << endl;
		delete treeWidget->takeTopLevelItem(treeWidget->indexOfTopLevelItem(treeWidget->currentItem()));
		profilesWereUpdated = true;
	}
	else
		cout << "warning : could not remove profile." << endl;
}

void ArchProfileManager::profileNew()
{
	ArchProfileEditor * ape = new ArchProfileEditor(this, 0);
	switch (ape->exec())
	{
		case QDialog::Accepted:
			profilesWereUpdated = true;
			profileListUpdate();
		 	break;
		case QDialog::Rejected:
			break;
		default:
			// should never be reached
			break;
	}
	/*
	return;
	
	
	
	bool ok;
	QString newName = QInputDialog::getText(this, QString("-- Arch Assistant --"),
											trUtf8("Choose a name for the new network profile :"),
											QLineEdit::Normal,
											QString(""),
											&ok);
	if (!ok || newName.isEmpty())
		return;

	bool okCopy = QFile::copy(
    						QString(":/network-profile-template"),//QString("/etc/network-profiles/template"),
    						QString("/etc/network-profiles/%1").arg(newName)
    						);
    if (!okCopy)
	{
		QMessageBox::warning(this, tr("Arch Assistant"),
                  trUtf8("Could not create new profile : an other file has same name"));
		return;
	}

    //treeWidget->addTopLevelItem();
    QTreeWidgetItem *itm = new QTreeWidgetItem(treeWidget);
	itm->setText(0, newName);
	itm->setText(1, QString("eth0"));
	itm->setText(2, QString("wlan"));
	
	profilesWereUpdated = true;
 
	TextEditor * te = new TextEditor(QString("/etc/network-profiles/%1").arg(newName));
	te->show();
	*/
}

// void ArchProfileManager::coucou()
// {
// 	cout << "coucou" << endl;
// }

void ArchProfileManager::profileRename()
{
	if (treeWidget->topLevelItemCount() < 1)
		return;
		
	bool ok;
	QString newName = QInputDialog::getText(this, QString("-- Arch Assistant --"),
                                         trUtf8("Rename profile '%1' :").arg(treeWidget->currentItem()->text(0)),
                                         QLineEdit::Normal,
                                         treeWidget->currentItem()->text(0),
                                         &ok);
    if (ok && !newName.isEmpty())
    {
		bool okRename = QFile::rename(
									QString("%1/%2").arg(netUtils->profilesDirName).arg(treeWidget->currentItem()->text(0)),
									QString("%1/%2").arg(netUtils->profilesDirName).arg(netUtils->checkProfileNameSyntax(newName))//QString("%1/%2").arg(netUtils->profilesDirName).arg(newName)
									);
		if (okRename)
		{
			treeWidget->currentItem()->setText(0,netUtils->checkProfileNameSyntax(newName));
			//parent->networkProfilesMenu->clear();
			//parent->networkProfilesMenuUpdate();
			
			//emit profilesUpdated();
			profilesWereUpdated = true;
		}
			
		else
			QMessageBox::warning(this, tr("Arch Assistant"),
                  trUtf8("Could not rename profile : an other file has same name"));
	}
}

void ArchProfileManager::profileCopy()
{
	if (treeWidget->topLevelItemCount() < 1)
		return;
		
	bool ok;
	QString newName = QInputDialog::getText(this, QString("-- Arch Assistant --"),
                                         trUtf8("Copy profile '%1' :").arg(treeWidget->currentItem()->text(0)),
                                         QLineEdit::Normal,
                                         QString("%1_copy").arg(treeWidget->currentItem()->text(0)),
                                         &ok);
    if (ok && !newName.isEmpty())
    {
		bool okCopy = QFile::copy(
									QString("%1/%2").arg(netUtils->profilesDirName).arg(treeWidget->currentItem()->text(0)),
									QString("%1/%2").arg(netUtils->profilesDirName).arg(netUtils->checkProfileNameSyntax(newName))//QString("%1/%2").arg(netUtils->profilesDirName).arg(newName)
									);
		if (okCopy)
		{
			profileListUpdate();//treeWidget->currentItem()->setText(0,newName);
			//parent->networkProfilesMenu->clear();
			//parent->networkProfilesMenuUpdate();
			
			//emit profilesUpdated();
			profilesWereUpdated = true;
		}
			
		else
			QMessageBox::warning(this, tr("Arch Assistant"),
                  trUtf8("Could not copy profile : an other file has same name"));
	}
}

void ArchProfileManager::profileConnect()
{
	profileConnectRequested = true;
	close();
}

void ArchProfileManager::profileDisconnect()
{
	//profileDisconnectRequested = true;
	profileDisconnectRequested = treeWidget->currentItem()->text(0);
	close();
}

void ArchProfileManager::profileListUpdate()
{
	treeWidget->clear();
	
	QStringList list = netUtils->getListOfProfiles();
	QStringList activeProfilesList = netUtils->getActiveProfilesList();

	for (int i = 0; i < list.size(); ++i)
	{
		QMap<QString, QString> testMap = profileGetInfosForProfile( QString("%1/%2").arg(netUtils->profilesDirName).arg(list.at(i)) );
			
		QTreeWidgetItem *itm = new QTreeWidgetItem(treeWidget);
		//itm->setFlags(Qt::ItemIsEditable | Qt::ItemIsUserCheckable);
		//itm->setCheckState(0,Qt::Unchecked);
		itm->setText(0, list.at(i));
		QFont font = itm->font(0);
		font.setBold(true);
		if (activeProfilesList.contains(list.at(i)))
			font.setUnderline(true);
		itm->setFont(0, font);
		itm->setToolTip(0, testMap["DESCRIPTION"]);
		itm->setText(1, testMap["INTERFACE"]);
		if (testMap["CONNECTION"].toLower() == "ethernet") //if (testMap["ESSID"].isEmpty())
		{
			itm->setText(2, "ETHERNET"); //itm->setText(2, "LAN");
			itm->setIcon(0,QIcon(":/cr22-action-nm_device_wired.png"));
		}
		else
		{
			itm->setText(2, "WIFI");
			itm->setIcon(0, QIcon(":/wifi_radar_22x22.png"));
		}
		if (activeProfilesList.contains(list.at(i)))
		{
			font.setBold(false);
			font.setUnderline(false);
			font.setItalic(true);
			itm->setText(3, tr("in use"));
			itm->setFont(3, font);
		}

// 		QTreeWidgetItem *after;
// 		after = treeWidget->childAt(1,1);
// 		after->setFlags(itm->flags() | Qt::ItemIsEditable);
		
		//treeWidget->setItemWidget(itm, 2,pbNew);
		//treeWidget->openPersistentEditor(itm, 2);
	}
	
	treeWidget->setSortingEnabled(true);
	treeWidget->sortByColumn(0, Qt::AscendingOrder);
	treeWidget->resizeColumnToContents ( 0 );
	treeWidget->resizeColumnToContents ( 1 );
	treeWidget->resizeColumnToContents ( 2 );
	treeWidget->resizeColumnToContents ( 3 );
}
//
