//
// C++ Implementation: configurationeditor
//
// Description: 
//
//
// Author: root <root@carrouf-G1>, (C) 2008
//
// Copyright: See COPYING file that comes with this distribution
//
//
#include "configurationeditor.h"
//#include "netutils.h"
#include <QtGui>

ConfigurationEditor::ConfigurationEditor(QWidget * parent, Qt::WFlags f, QString fileName)
	: QDialog(parent, f)
{
	configFileName = fileName;
	treeWidget = new QTreeWidget(this);
	treeWidget->setColumnCount(2);
	treeWidget->setRootIsDecorated(false);
	treeWidget->setAlternatingRowColors(true);
	treeWidget->setSelectionMode(QAbstractItemView::NoSelection);
	
	QStringList labelsList;
	labelsList << tr("Key name") << tr("Options");
	treeWidget->setHeaderLabels(labelsList);
	
	QPushButton *pbClose = new QPushButton(trUtf8("&Cancel"), this);
	QPushButton *pbOk = new QPushButton(trUtf8("&Save"), this);
	pbOk->setIcon(QIcon(":/filesave.png"));
	pbClose->setIcon(QIcon(":/fileclose.png"));
	
	QHBoxLayout *layout = new QHBoxLayout;
	layout->addStretch();
	layout->addWidget( pbOk );
	layout->addWidget( pbClose );
	//layout->addLayout( layoutV );
	
	QVBoxLayout *layoutV = new QVBoxLayout;
	//layoutV->addStretch();
	layoutV->addWidget( treeWidget );
	layoutV->addLayout( layout );
	
	initWidgets();
	loadConfiguration();
	updateTreeWidgetItems();
	
	treeWidget->resizeColumnToContents (0);
	treeWidget->resizeColumnToContents (1);
	
	//setSizeGripEnabled(true);
	setLayout( layoutV );
	setModal(true);
	setWindowTitle ( tr("Arch Assistant - Configuration editor") );
	setWindowIcon(QIcon(":/archlogo.png"));
	setMinimumSize(600, 480);
	
	connect( pbOk, SIGNAL( clicked() ), this, SLOT( saveConfiguration() ) );
	connect( pbClose, SIGNAL( clicked() ), this, SLOT( reject() ) );
}

void ConfigurationEditor::updateTreeWidgetItems()
{
	QTreeWidgetItem *itm ;
	
	itm = new QTreeWidgetItem(treeWidget);
	itm->setText(0, tr("Autoconnect on startup"));
	treeWidget->setItemWidget(itm, 1, cbAutoconnect);
	
	itm = new QTreeWidgetItem(treeWidget);
	itm->setText(0, tr("Single profile at once"));
	treeWidget->setItemWidget(itm, 1, cbSingleProfileAtOnce);
	
	itm = new QTreeWidgetItem(treeWidget);
	itm->setText(0, tr("Force wlan device"));
	treeWidget->setItemWidget(itm, 1, leWlanName);
	
	itm = new QTreeWidgetItem(treeWidget);
	itm->setText(0, tr("Downloader applications list"));
	treeWidget->setItemWidget(itm, 1, leDownloaderApps);
}

void ConfigurationEditor::initWidgets()
{
	QStringList sl;

	cbAutoconnect = new QComboBox();
	cbSingleProfileAtOnce = new QComboBox();
	leWlanName = new QLineEdit();
	leDownloaderApps = new QLineEdit();
	
	sl.clear();
	sl << "yes" << "no";
	cbAutoconnect->addItems(sl);
	cbSingleProfileAtOnce->addItems(sl);
}

void ConfigurationEditor::loadConfiguration()
{
	QString wlanDeviceName;
	QString autoconnectOnStartup;
	QString singleProfileAtOnce;
	QString downloaderApps;
	
	QSettings settings(configFileName, QSettings::IniFormat);
	settings.beginGroup("MAIN");
	//pacmanBin = settings.value("pacman", "pacman").toString();
	//shellBin = settings.value("terminal", "konsole").toString();
	autoconnectOnStartup = settings.value("autoconnect", "no").toString();
	wlanDeviceName = settings.value("wlan", "").toString();
	singleProfileAtOnce = settings.value("single_profile_at_once", "no").toString();
	downloaderApps = settings.value("downloader_apps", "").toString();
	settings.endGroup();
	
	//qDebug(qPrintable(autoconnectOnStartup));
	
	if (autoconnectOnStartup.toLower() == "true" || autoconnectOnStartup.toLower() == "yes" || autoconnectOnStartup.toLower() == "1" )
		cbAutoconnect->setCurrentIndex(0);
	else
		cbAutoconnect->setCurrentIndex(1);
	
	if (singleProfileAtOnce.toLower() == "true" || singleProfileAtOnce.toLower() == "yes" || singleProfileAtOnce.toLower() == "1" )
		cbSingleProfileAtOnce->setCurrentIndex(0);
	else
		cbSingleProfileAtOnce->setCurrentIndex(1);
	
	leWlanName->setText(wlanDeviceName);
	leDownloaderApps->setText(downloaderApps);
}

void ConfigurationEditor::saveConfiguration()
{
	switch (
			QMessageBox::question(
								this,
								tr("Confirm configuration write"),
								tr("Save settings ?"),
								tr("&Ok"), tr("&Cancel"),
								0, 1)
			)
	{
		case 1:
			return;
	}
	
	QSettings settings(configFileName, QSettings::IniFormat);
	settings.beginGroup("MAIN");
	//settings.setValue( "pacman", pacmanBin );
	//settings.setValue( "terminal", shellBin );
	settings.setValue( "wlan", leWlanName->text() );
	settings.setValue( "downloader_apps", leDownloaderApps->text() );
	settings.setValue( "autoconnect", cbAutoconnect->currentText() );
	settings.setValue( "single_profile_at_once", cbSingleProfileAtOnce->currentText() );
	settings.endGroup();
	
	accept();
}

// ConfigurationEditor::~ConfigurationEditor()
// {
// }


