//
// C++ Implementation: netutils
//
// Description: 
//
//
// Author: root <root@carrouf-G1>, (C) 2007
//
// Copyright: See COPYING file that comes with this distribution
//
//

#include "netutils.h"
#include <QtCore>
//#include <QtGui>

NetUtils::NetUtils()
{
	profilesDirName = "/etc/network.d";
	profilesRunningDirName = "/var/run/network/profiles";
	netcfgBinary = "sudo netcfg2";
	ifconfigBinary = "sudo ifconfig";
	iwlistBinary = "sudo iwlist";
	//iwconfigBinary = "iwconfig";
}

QString NetUtils::guessEssid()
{
	QProcess *ifconfigProcess = new QProcess();
	QString line = "ERROR";
	QString ssid;

	//ifconfigProcess->start("iwlist eth1 scan | grep ESSID | sed 's/.*ESSID:\"\([^\"]\\+\\)\".*/\1/'");
	ifconfigProcess->start(iwlistBinary + " " + getWifiInterfaceName() + " scan");
	//iwlist $IFACE scan | grep ESSID | sed 's/.*ESSID:"\([^"]\+\)".*/\1/'
	if (!ifconfigProcess->waitForStarted())
		return line;
	if (!ifconfigProcess->waitForFinished())
		return line;

	QTextStream stream(ifconfigProcess->readAll().data());
	
	do {
		line = stream.readLine().trimmed();
		//qDebug() << "'" << qPrintable(line) << "'"<< endl;
		if (line.startsWith("ESSID:"))
		{
			// extract essid name from cell
			ssid = line.mid ( line.indexOf ( "\"" ) + 1, line.length() - line.indexOf ( "\"" ) - 2 );
			if ((ssid=="") || (ssid==" "))
				ssid = QObject::trUtf8("(hidden cell)");//ssid = trUtf8("(hidden cell)");
			break;
		}
			
	} while (!line.isNull());
	
	return ssid;//return line;
}

QString NetUtils::getHostname()
{
	QString currentHostName = "";
	QProcess *getHostNameProcess = new QProcess();
	getHostNameProcess->setProcessChannelMode(QProcess::MergedChannels);
	getHostNameProcess->start("hostname");
	
	if (!getHostNameProcess->waitForStarted())
	{
		qDebug("error getCurrentHostName");
		return currentHostName;
	}			
	if (!getHostNameProcess->waitForFinished())
	{
		qDebug("error getCurrentHostName");
		return currentHostName;
	}
	
	QTextStream stream(getHostNameProcess->readAll());
	//QTextStream stream(&getHostNameProcess);
	currentHostName = stream.readLine().trimmed();
	
	return currentHostName;
}

QString NetUtils::getCurrentUserName()
{
	QString currentUserName = "";
	QProcess *getUserNameProcess = new QProcess();
	getUserNameProcess->setProcessChannelMode(QProcess::MergedChannels);
	getUserNameProcess->start("whoami");
	
	if (!getUserNameProcess->waitForStarted())
	{
		qDebug("error getCurrentUserName");
		return currentUserName;
	}			
	if (!getUserNameProcess->waitForFinished())
	{
		qDebug("error getCurrentUserName");
		return currentUserName;
	}
	
	QTextStream stream(getUserNameProcess->readAll());
	//QTextStream stream(&getHostNameProcess);
	currentUserName = stream.readLine().trimmed();
	
	return currentUserName;
}

QString NetUtils::getWifiInterfaceName()
{
	QString wifiInterfaceName = "";
	QDir dir("/sys/class/net");
	dir.setFilter(QDir::AllDirs);
	QStringList list = dir.entryList();
	
	for (int i=0 ; i < list.size() ; i++)
	{
		//qDebug() << "'" << qPrintable(list.at(i)) << "'"<< endl;
		if ( dir.exists( list.at(i) + "/wireless") )//if (QFile::exists(dir.dirName() + "/" + list.at(i) + "/wireless"))
		{
			wifiInterfaceName = list.at(i);
			break;
		}
	}
	
	return wifiInterfaceName;
}

bool NetUtils::getInterfaceWifiStatus(const QString & interfaceName)
{
	bool status = false;
	QDir dir("/sys/class/net/" + interfaceName + "/wireless");
	if (dir.exists())
		status = true;
	
	return status; //false = ethernet ; true = wireless
}

QString NetUtils::getInterfaceStatus(QString interfaceName)
{
	QString status = "";
	QFile file("/sys/class/net/" + interfaceName + "/operstate");
	if (file.exists())
	{
		if (!file.open(QIODevice::ReadOnly | QIODevice::Text))
			return status;

		//while (!file.atEnd()) {
			QByteArray line = file.readLine();
			status = line.trimmed();
		//}
	}
	
	return status;
}

QString NetUtils::getInterfaceType(QString interfaceName)
{
	QString status = "";
	QFile file("/sys/class/net/" + interfaceName + "/type");
	if (file.exists())
	{
		if (!file.open(QIODevice::ReadOnly | QIODevice::Text))
			return status;

		//while (!file.atEnd()) {
		QByteArray line = file.readLine();
		status = line.trimmed();
		//}
	}
	
	return status;
}

QString NetUtils::getEssidForProfile(QString fileName, QString interfaceName)
{
	QString str = "";
	QString strEssid = "";
	QString strAutoconnect = "";
	QString strInterfaceName = "";
	QString file = QString("%1/%2").arg(profilesDirName).arg(fileName);
	
	if ( QFile::exists(file) )
	{
		QSettings settings(file, QSettings::IniFormat);
		
		strAutoconnect = settings.value("AUTOCONNECT", "").toString();
		strAutoconnect = strAutoconnect.left(strAutoconnect.lastIndexOf("#")).trimmed(); // remove commentary if present
		strInterfaceName = settings.value("INTERFACE", "").toString();
		strInterfaceName = strInterfaceName.left(strInterfaceName.lastIndexOf("#")).trimmed(); // remove commentary if present
		strEssid = settings.value("ESSID", "").toString();
		strEssid = strEssid.left(strEssid.lastIndexOf("#")).trimmed(); // remove commentary if present
		
		//qDebug() << "level zero  = " << qPrintable(strEssid.trimmed()) << endl;		
		//qDebug() << qPrintable(interfaceName) <<  qPrintable(strInterfaceName) << endl;
		
		if (interfaceName == strInterfaceName)
		{
			if (strEssid.trimmed().length() > 0 && strAutoconnect.toLower() != "no") // only transmit essid if autoconnect key != no
			{
				str = strEssid.trimmed();
			}
		}
	}

	return str;
}

QString NetUtils::getConnectionForProfile(const QString & fileName)
{
	QString str = "";
	QString file = QString("%1/%2").arg(profilesDirName).arg(fileName);
	
	if ( QFile::exists(file) )
	{
		QSettings settings(file, QSettings::IniFormat);
		str = settings.value("CONNECTION", "").toString();
		str = str.left(str.lastIndexOf("#")).trimmed(); // remove commentary if present
	}

	return str;
}


// QString NetUtils::getAutoconnectForProfile(QString fileName)
// {
// 	QString str = "";
// 	QString file = QString("%1/%2").arg(profilesDirName).arg(fileName);
// 	
// 	if ( QFile::exists(file) )
// 	{
// 		QSettings settings(file, QSettings::IniFormat);
// 		str = settings.value("AUTOCONNECT", "").toString();
// 		str = str.left(str.lastIndexOf("#")).trimmed(); // remove commentary if present
// 	}
// 
// 	return str;
// }

QStringList NetUtils::getListOfProfiles()
{
	QStringList listOfProfiles;
	
	QDir dir(profilesDirName);
	dir.setFilter(QDir::Files);
	
	QFileInfoList list = dir.entryInfoList();
	
	for (int i = 0; i < list.size(); ++i)
	{
		QFileInfo fileInfo = list.at(i);
		if (!fileInfo.fileName().endsWith("~"))
			listOfProfiles << fileInfo.fileName();
	}
	
	return listOfProfiles;
}

int NetUtils::getRunningInterfacesCount()
{
	// Get list of names of active network interfaces
	
	int count = 0;
	QProcess *ifconfigProcess = new QProcess();

	ifconfigProcess->start(ifconfigBinary + " -a");
	if (!ifconfigProcess->waitForStarted())
		return count;
	if (!ifconfigProcess->waitForFinished())
		return count;

	QTextStream stream(ifconfigProcess->readAll().data());
	QString line;
	
	do {
		line = stream.readLine();
		if (line.contains("RUNNING"))
			++count;
		if (line.startsWith("lo"))
			break;
	} while (!line.isNull());
	
	return count;
}

QStringList NetUtils::getInterfacesList()
{
	QStringList ethList;
	QDir dir("/sys/class/net");
	dir.setFilter(QDir::AllDirs);
	QStringList list = dir.entryList();
	
	for (int i=0 ; i < list.size() ; i++)
	{
		//qDebug() << "'" << qPrintable(list.at(i)) << "'"<< endl;
		if ( list.at(i) != "." && list.at(i) != ".." )//if ( list.at(i) != "lo" && list.at(i) != "." && list.at(i) != ".." )
		{
			if ( getInterfaceType(list.at(i)) == "1" )
				ethList << list.at(i);
		}
	}
	
	return ethList;
}

QStringList NetUtils::getActiveProfilesList()
{
	QStringList profilesList;
	
	QDir dir(profilesRunningDirName);
	dir.setFilter(QDir::Files);
	QStringList list = dir.entryList();
	
	for (int i=0 ; i < list.size() ; i++)
	{
		//qDebug() << "'" << qPrintable(list.at(i)) << "'"<< endl;
		profilesList << list.at(i);
// 		if ( list.at(i) != "." && list.at(i) != ".." )//if ( list.at(i) != "lo" && list.at(i) != "." && list.at(i) != ".." )
// 		{
// 			if ( getInterfaceStatus(list.at(i)) != "unknown" )
// 				profilesList << list.at(i);
// 		}
	}
	
	return profilesList;
}

QString NetUtils::checkProfileNameSyntax(const QString & profileText)
{
	QString allowedChars="0123456789_-.AZERTYUIOPQSDFGHJKLMWXCVBNazertyuiopqsdfghjklmwxcvbn";
	QString cleanText = "";//QString cleanText = profileText;
	
	if ( profileText.size() > 0 )
	{
		//if ( !allowedChars.contains( profileText.right(1) ) )
		//	cleanText = profileText.left( profileText.size()-1 );
		for (int i = 0; i < profileText.size(); i++)
		{
			if ( allowedChars.contains( profileText.at(i) ) )
				cleanText.append( profileText.at(i) );
		}
	}
	
	return cleanText;
}

QString NetUtils::getVendorNameForInterface(const QString & interfaceName)
{
	QString status = interfaceName;
	QString line;
	QSettings settings("/sys/class/net/" + interfaceName + "/device/uevent", QSettings::IniFormat);

	QString pciAddress = settings.value("PCI_SLOT_NAME", "").toString();
	if ( !pciAddress.isEmpty() )
	{
		pciAddress = pciAddress.right(7);//qDebug(qPrintable(status));
		
		QProcess *grepProcess = new QProcess();
		grepProcess->setProcessChannelMode(QProcess::MergedChannels);
		grepProcess->start("lspci");
		
		if (!grepProcess->waitForStarted())
			return interfaceName;
		if (!grepProcess->waitForFinished())
			return interfaceName;
		
		QTextStream stream(grepProcess->readAll().data());
	
		do
		{
			line = stream.readLine().trimmed();
			if (line.startsWith(pciAddress))
			{
				status = line.mid(8);
				break;
			}
		} while (!line.isNull());
	}		
		
	return status;
}

QMap<int, QStringList> NetUtils::parseIwlistScan(QString buffer)
{
	QMap<int, QStringList> map;
	QString mac_address;
	QString line;
	QString ssid;
	QString channel;
	QString quality;
	QString keyOnOff;
	QString mode;
	QString protocol;

	QStringList bufferLines = buffer.split("<<#>>");
	
	if (bufferLines.size() > 0)
	{
		if ( bufferLines.at(0).contains("No scan results") )
			return map;
	}

	//do {
	for (int i=0 ; i < bufferLines.size() ; i++)
	{
		mac_address = "";
		ssid = "default";
		channel = "n/a";
		quality = "default";
		keyOnOff = "off";
		mode = "default";
		protocol = "WEP";
		QStringList marcel = bufferLines.at(i).split("<<>>");
		
		//line = stream.readLine().trimmed();
		//qDebug() << qPrintable(line) << "//" << endl;

		for (int j=0 ; j < marcel.size() ; j++)
		{
			line = marcel.at(j);
			//qDebug() << "line:" << qPrintable(line) << "__" << endl;
			
			if ( line.startsWith ( "Cell" ) && line.contains( "Address:" ) )
			{
				QStringList tmp = line.split("Address:");
				mac_address = tmp.at(tmp.size()-1).trimmed();
				//qDebug() << "mac_address:" << qPrintable(mac_address) << "//" << endl;
			}

			if ( line.startsWith ( "ESSID:" ) )
			{
				//ssid = line.mid ( line.indexOf ( "\"" ) + 1, line.lastIndexOf ( "\"" ) - 1 );
				ssid = line.mid ( line.indexOf ( "\"" ) + 1, line.lastIndexOf ( "\"" ) - line.indexOf ( "\"" ) - 1 );
				if ((ssid=="") || (ssid==" "))
					ssid = QString("(hidden cell)");
				//qDebug() << "ssid:" << qPrintable(ssid) << "//" << endl;
			}
			
			if ( line.startsWith ( "Encryption key:on" ) )
				keyOnOff = "on";

			if ( line.startsWith ( "Quality:" ) )
			{
				quality = line.mid ( line.indexOf (":") + 1, line.indexOf ("/") - line.indexOf (":") - 1 );
				//qDebug() << "quality:" << qPrintable(quality) << "//" << endl;
			}

			if ( line.startsWith ( "Quality=" ) )
			{
				quality = line.mid (line.indexOf ("=") + 1, line.indexOf ("/") - line.indexOf ("=") - 1 );
				//qDebug() << "quality:" << qPrintable(quality) << "//" << endl;
			}
			
			if ( line.startsWith("Channel:") )
			{
				channel = line.mid ( line.indexOf (":") + 1, line.size());
				//qDebug() << "channel:" << qPrintable(line.mid ( line.indexOf (":") + 1, line.size() )) << "//" << endl;
			}
			
			if ( line.startsWith("Mode:") )
			{
				if ( line.contains ( "Master" ) )
					mode = "Managed";//itm->setText(3, tr("Managed"));
				if ( line.contains ( "Ad-Hoc" ) )
					mode = "Ad-Hoc";//itm->setText(3, tr("Ad-Hoc"));
			}

			if ( line.startsWith ( "IE:" ) )
			{
				if ( line.toUpper().contains("WPA") )
					protocol = "WPA";//itm->setText(4, "WPA");
				else
					protocol = "WEP";//itm->setText(4, "WEP");
			}
		}

		// fill map entries for current cell
		map[i] << ssid << channel << quality << keyOnOff << mode << protocol << mac_address;
	
	}// while (!line.isNull());

	return map;
}

// NetUtils::~NetUtils()
// {
// }


